@@ -227,7 +227,7 @@ def hcascade(tables, colnames=None):
             if any(empty): tab.remove_columns(np.array(cols)[empty])
 
         cols=find_colnames(tab,name)#[ colname for colname in tab.colnames if name in colname]
-        tab.rename_columns(cols, ["%s_%d"%(name,i+1) for i in range(len(cols))])
+        if cols: tab.rename_columns(cols, ["%s_%d"%(name,i+1) for i in range(len(cols))])
     return tab
 
 def extnames(hdulist): return list(ext.name for ext in hdulist)
